        title   "QBUS-POR"
;++
; QBUS Reset and 60 Hz clock emulation
;
; Brad Parker <brad@heeltoe.com> 8/2012
; Based on the original circuit by John Wilson, D Bit <wilson@dbit.com>.
; 
; 10-pin header on back of H9270 backplane:
;
; 1- BPOK H
; 2- BEVNET L
; 3- SRUN L
; 4- Key
; 5- Gnd
; 6- CL3 L
; 7- CS3 L
; 8- Spare
; 9- BHALT L
; 10- DCOK H
;
; looking at the bottom of a H9270
; 10 BDCOK      BHALT   9
; 8  (spare)    CS3 L   7
; 6  CL3 L      GND     5
; 4  (key)      SRUN    3
; 2  BEVNT      BPOK    1
;
; PIC16F84:
;
; pic pin 17 (ra0) -> BDCOK
; pic pin 18 (ra1) -> BPOK      toggle w/pb
; pic pin 1  (ra2) -> BHALT     toggle w/pb
; pic pin 2  (ra3) -> 
; pic pin 3  (ra4) -> BEVNT
;
; pic pin 13 (rb7) -> led
; pic pin 12 (rb6) -> led
; pic pin 8  (rb2) <- SRUN
; pic pin 7  (rb1) <- pb to gnd
; pic pin 6  (rb0) <- pb to gnd
;
; Crystal:
; 
; 32768hz
; clk/4 = 8192hz
; 8192 / 136 = 60hz
; program prescaler to be / 136 or (136/2)=68 for 30Hz
;
;--
        list    p=16f84
        include <p16f84.inc>
        __config _LP_OSC&_WDT_OFF ; low-power oscillator (watch xtal)
;
hz      equ     d'60'           ; output frequency in Hz
clk     equ     d'32768'        ; input clock frequency in Hz (watch xtal)
;
cyc     equ     clk/4           ; 4 clocks per cycle
div     equ     cyc/(hz*2)      ; clock divider
cnt     equ     d'256'-(div/2)  ; actual value for TMR0
;
        cblock  h'0C'           ; variables (start at 0x0c in ram)
         state
         dc_delay
         halt_delay
         status_temp
         w_temp
        endc

        org     0x000
        goto    start           ; reset vector
;;
;;  Interrupt service routine
;; 
        org     0x004
interrupt:                      ; interrupt vector
        movwf   w_temp          ; save state
        swapf   STATUS,w
        movwf   status_temp
isr:
        bcf     INTCON,T0IF     ; reset timer interrupt

        movfw   TMR0
        addlw   cnt             ; adjust count for current timer
        movwf   TMR0

        movlw   0x10
        xorwf   PORTA,f         ; toggle bit 4

isr_done:
        swapf   status_temp,w   ; restore
        movwf   STATUS
        swapf   w_temp,f
        swapf   w_temp,w
        retfie

;;
;; Initialization
;; 
start:
        bcf     STATUS,RP0      ; select low register space
        clrf    PORTA           ; clear output latch
        clrf    PORTB
        
        bsf     STATUS,RP0      ; select high bank

        movlw   0x00
        movwf   TRISA           ; PORTA is all outputs

        movlw   0x3f
        movwf   TRISB           ; PORTB has input

        bcf     OPTION_REG,7    ; enable weak pullups on port B
        bcf     OPTION_REG,T0CS ; timer0 input from clock
        bcf     OPTION_REG,PSA  ; timer0 prescaler
        bcf     OPTION_REG,PS0
        bcf     OPTION_REG,PS1
        bcf     OPTION_REG,PS2

        bcf     STATUS,RP0      ; back to low bank

        movlw   cnt
        movwf   TMR0            ; set up timer0
        
        bcf     INTCON,T0IF
        bsf     INTCON,T0IE     ; enable interrupts from timer
        
        bsf     PORTA,0         ; DCOK
        bsf     PORTA,1         ; POK
        bsf     PORTA,2         ; HALT
        bcf     PORTA,4		; event

        bcf     PORTB,6         ; led0
        bsf     PORTB,7         ; led1

        movlw   0xff           ; power on reset pulse
        movwf   dc_delay
        clrf    halt_delay
        clrf    state
        
        btfss   PORTB,2         ; check SRUN
        goto    ints_on
        bcf     PORTB,7         ; led1 on
        bsf     state,2

        bsf     state,0         ; force pok de-assert
        
ints_on:
        bsf     INTCON,GIE      ; enable interrupts

;
; Main loop
;   manage POK and HALT pulses
;   poll push buttons
;   poll SRUN
; 
; 60Hz clock is toggled in ISR
;
loop:
        ;; DCOK pulse
        btfss   state,0
        goto    loop1

        bcf     PORTA,0         ; keep DCOK off
        decfsz  dc_delay,f
        goto    loop1
        bsf     PORTA,0         ; turn DCOK back on
        bcf     state,0
        bcf     PORTB,6         ; led0
        
        ;; HALT pulse
loop1:
        btfss   state,1
        goto    loop2

        bcf     PORTA,2         ; keep HALT off
        decfsz  halt_delay,f
        goto    loop2
        bsf     PORTA,2         ; turn HALT back on
        bcf     state,1

        ;; track push buttons
loop2:
        btfss   PORTB,0         ; check RB0 - pb
        call    push_pb0
        
        btfss   PORTB,1         ; check RB1 - pb
        call    push_pb1

        ;; track SRUN and set led
        btfss   PORTB,2         ; check SRUN
        goto    loop3
        btfsc   state,2         ; SRUN ON - previous?
        goto    loop            ; on and previously on - done
        
        bcf     PORTB,7         ; prev=0, led1 on
        bsf     state,2
        goto    loop4
loop3:                          ; it's off
        btfsc   state,2         ; SRUN OFF - previous?
        bsf     PORTB,7         ; prev=1, led1 off
        bcf     state,2
loop4:
        goto    loop            ; back to top

;
toggle_ra4
        btfsc   PORTA,4
        b       toggle_ra4_off  ; it was on
        bsf     PORTA,4         ; set RA4
        return
toggle_ra4_off
        bcf     PORTA,4         ; clear RA4
        return

;
push_pb0
        movlw   0xff
        movwf   dc_delay
        bsf     state,0
        bsf     PORTB,6         ; led0
        return

push_pb1
        movlw   0xff
        movwf   halt_delay
        bsf     state,1
        return
        
        end

